/*
 * jidlink.h - establish a link between Jabber IDs
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

 /*
   TODO:

   manager: ask before accepting
   ability to read/write to a jidlink
   canceling a jidlink, and sending appropriate error to the peer
   if a jidlinkhandler fails, then the jidlink should delete it and go idle.

 */
 
#ifndef JABBER_JIDLINK_H
#define JABBER_JIDLINK_H

#include<qobject.h>
#include<qdom.h>
#include<qstring.h>
#include"jid.h"
#include"client.h"

namespace Jabber
{
	class JidLinkHandler : public QObject
	{
		Q_OBJECT
	public:
		JidLinkHandler(QObject *parent=0);
		virtual ~JidLinkHandler()=0;

		virtual Jid peer() const;
		virtual void close();
		virtual bool isConnected() const;

		virtual void write(const QByteArray &);
		virtual QByteArray read();
		virtual bool canRead() const;

	signals:
		void connected();
		void connectionClosed();
		void delayedCloseFinished();
		void readyRead();
		void bytesWritten(int);
		void error(int);

	private:
		class Private;
		Private *d;
	};

	class JidLink : public QObject
	{
		Q_OBJECT
	public:
		enum { Idle, Negotiating, Establishing, Connected, Closing, Waiting };
		enum { ErrNeg, ErrStream };
		JidLink(Client *client);
		~JidLink();

		bool connectToJid(const Jid &jid, const QString &key, const QString &method="");
		void accept();

		Jid peer() const;
		QString key() const;
		QString method() const;
		int state() const;

	signals:
		void error(int);

	private slots:
		void neg_finished();
		void js_finished();
		void jsh_connectionClosed();

	private:
		class Private;
		Private *d;

		void getMethod();
		void getStream();
		void doEstablish();

		friend class JidLinkManager;
		void wait(const Jid &, const QString &);
		void connectSuccess(JidLinkHandler *);
		void connectError();
		void incomingJSHandler(JidLinkHandler *);
	};

	typedef QPtrList<JidLink> JidLinkList;
	typedef QPtrListIterator<JidLink> JidLinkListIt;
	class JidLinkManager : public QObject
	{
		Q_OBJECT
	public:
		JidLinkManager(Client *);
		~JidLinkManager();

		QStringList supportedMethods() const;
		const JidLinkList *streamList() const;
		void takeIncomingJSHandler(const QString &key, JidLinkHandler *);

	private slots:
		void pjs_incoming(const Jid &from, const QString &id, const QString &key);
		void dtcp_connected();
		void dtcp_error(int);

	private:
		class Private;
		Private *d;

		friend class JidLink;
		void link(JidLink *);
		void unlink(JidLink *);
		void doEstablish(JidLink *);
		JidLink *findStream(const Jid &, const QString &) const;
	};

	class JT_JidLink : public Task
	{
		Q_OBJECT
	public:
		JT_JidLink(Task *);
		~JT_JidLink();

		void set(const Jid &, const QString &key);

		void onGo();
		bool take(const QDomElement &);

		Jid jid() const;

	private:
		QDomElement iq;
		Jid to;
		QString v_key;
	};

	class JT_PushJidLink : public Task
	{
		Q_OBJECT
	public:
		JT_PushJidLink(Task *);
		~JT_PushJidLink();

		bool take(const QDomElement &);

		void respondSuccess(const Jid &to, const QString &id);
		void respondError(const Jid &to, const QString &id, int code, const QString &str);

	signals:
		void incoming(const Jid &from, const QString &id, const QString &key);
		void error(const Jid &from, const QString &key, int code, const QString &str);
	};

	class JT_Negotiate : public Task
	{
		Q_OBJECT
	public:
		JT_Negotiate(Task *);
		~JT_Negotiate();

		void get(const Jid &, const QString &, const QStringList &);

		void onGo();
		bool take(const QDomElement &);

		Jid jid() const;
		QString option() const;

	private:
		QDomElement iq;
		Jid to;
		QString v_option;
	};

	class JT_PushNegotiate : public Task
	{
		Q_OBJECT
	public:
		JT_PushNegotiate(Task *);
		~JT_PushNegotiate();

		bool take(const QDomElement &);

	private:
		QString getPreferred(const QString &, const QStringList &);
	};
}

#endif
