/*
 * psiaccount.h - handles a Psi account
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef PSIACCOUNT_H
#define PSIACCOUNT_H

#include<qlabel.h>
#include<qptrlist.h>
#include"kmp/jid.h"
#include"contactview.h"

namespace Jabber
{
	class Status;
	class RosterItem;
	class Client;
	class StreamError;
	class Resource;
	class Message;
	class DTCPConnection;
};

using namespace Jabber;

class PsiCon;
class PsiAccount;
class PsiEvent;
class UserAccount;
class ContactProfile;
class QWidget;
class EventQueue;
class UserListItem;
class UserList;
class EventDlg;
class ChatDlg;
class EDB;

class AccountLabel : public QLabel
{
	Q_OBJECT
public:
	AccountLabel(PsiAccount *, QWidget *parent=0, bool simpleMode=false);
	~AccountLabel();

private slots:
	void updateName();
	void deleteMe();

private:
	PsiAccount *pa;
	bool simpleMode;
};

class PsiAccount : public QObject
{
	Q_OBJECT
public:
	PsiAccount(const UserAccount &acc, PsiCon *parent);
	~PsiAccount();

	bool isActive() const;
	bool isConnected() const;
	const QString &name() const;

	const UserAccount & userAccount() const;
	void setUserAccount(const UserAccount &);
	const Jid & jid() const;
	QString nameWithJid() const;

	Jabber::Client *client() const;
	ContactProfile *contactProfile() const;
	EventQueue *eventQueue() const;
	EDB *edb() const;
	PsiCon *psi() const;

	Status status() const;
	void setStatusDirect(const Status &);
	void login();
	void logout(bool fast=false);
	bool loggedIn() const;
	void setNick(const QString &);
	QString nick() const;

	QWidget *dialogFind(const char *className, const Jid &jid="");
	void dialogRegister(QWidget *w, const Jid &jid="");
	void dialogUnregister(QWidget *w);

	void modify();
	void changeVCard();
	void changePW();
	void changeStatus(int);
	void doManageServices();

	void showXmlConsole();
	void openAddUserDlg();
	void openGroupChat(const Jid &);
	bool groupChatJoin(const QString &host, const QString &room, const QString &nick);
	void groupChatSetStatus(const QString &host, const QString &room, const Status &);
	void groupChatLeave(const QString &host, const QString &room);

	UserListItem *find(const Jid &) const;
	QPtrList<UserListItem> findRelavent(const Jid &) const;
	UserListItem *findFirstRelavent(const Jid &) const;
	UserList *userList() const;

	bool checkConnected(QWidget *parent=0);
	void playSound(const QString &);

	QLabel *accountLabel(QWidget *parent=0, bool simpleMode=false);
	QStringList hiddenChats(const Jid &) const;

signals:
	void disconnected();
	void updatedActivity();
	void updatedAccount();
	void queueChanged();
	void updateContact(const UserListItem &);
	void updateContact(const Jid &);
	void updateContact(const Jid &, bool);
	void nickChanged();

public slots:
	void setStatus(const Status &);

	void secondsIdle(int);
	void openNextEvent();

	void dj_sendMessage(const Message &);
	void dj_composeMessage(const Jid &, const QString &);
	void dj_addAuth(const Jid &);
	void dj_add(const Jid &, const QString &, const QStringList &, bool authReq);
	void dj_authReq(const Jid &);
	void dj_auth(const Jid &);
	void dj_deny(const Jid &);
	void dj_rename(const Jid &, const QString &);
	void dj_remove(const Jid &);

	void actionDefault(const Jid &);
	void actionRecvEvent(const Jid &);
	void actionSendMessage(const Jid &);
	void actionSendMessage(const JidList &);
	void actionSendUrl(const Jid &);
	void actionRemove(const Jid &);
	void actionRename(const Jid &, const QString &);
	void actionGroupRename(const QString &, const QString &);
	void actionHistory(const Jid &);
	void actionStatusShow(const Jid &);
	void actionOpenChat(const Jid &);
	void actionOpenChatSpecific(const Jid &);
	void actionAgentSetStatus(const Jid &, int);
	void actionInfo(const Jid &);
	void actionAuth(const Jid &);
	void actionAuthRequest(const Jid &);
	void actionAuthRemove(const Jid &);
	void actionAdd(const Jid &);
	void actionGroupAdd(const Jid &, const QString &);
	void actionGroupRemove(const Jid &, const QString &);
	void actionHistoryBox(PsiEvent *);
	void actionTest(const Jid &);
	void actionRegister(const Jid &);
	void actionSearch(const Jid &);

private slots:
	void client_connected();
	void client_handshaken();
	void client_error(const StreamError &);
	void client_closeFinished();
	void client_authFinished(bool, int, const QString &);
	void client_rosterRequestFinished(bool, int, const QString &);
	void client_rosterItemUpdated(const RosterItem &);
	void client_rosterItemRemoved(const RosterItem &);
	void client_resourceAvailable(const Jid &, const Resource &);
	void client_resourceUnavailable(const Jid &, const Resource &);
	void client_presenceError(const Jid &, int, const QString &);
	void client_messageReceived(const Message &);
	void client_subscription(const Jid &, const QString &);
	void client_debugText(const QString &);
	void client_groupChatJoined(const Jid &);
	void client_groupChatLeft(const Jid &);
	void client_groupChatPresence(const Jid &, const Status &);
	void client_groupChatError(const Jid &, int, const QString &);
	void client_incomingDTCPConnection(DTCPConnection *);

	void reconnect();
	void enableNotifyOnline();

	void slotClientVersionFinished();

	void processReadNext(const Jid &);
	void processReadNext(const UserListItem &);
	void processChats(const Jid &);
	void chatMessagesRead(const Jid &);

	void slotCheckVCard();
	void edb_finished();

private:
	class Private;
	Private *d;

	void deleteAllDialogs();
	void stateChanged();
	void simulateContactOffline(UserListItem *);
	void simulateRosterOffline();
	void cpUpdate(const UserListItem &, const QString &rname="", bool fromPresence=false);
	void logEvent(const Jid &, PsiEvent *);
	void handleEvent(PsiEvent *);
	void queueEvent(PsiEvent *);
	void openNextEvent(const UserListItem &);
	void updateReadNext(const Jid &);
	void openChat(const Jid &);
	EventDlg *ensureEventDlg(const Jid &);
	ChatDlg *ensureChatDlg(const Jid &);

	bool isDisconnecting, notifyOnlineOk, doReconnect, usingAutoStatus, presenceSent, v_isActive;
};

#endif
